"use strict";
/*! *****************************************************************************
Copyright (c) JDJR Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0
Author: kemao 2018-11-29 18:29:00

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
exports.__esModule = true;
var fs = require("fs-extra");
var LarkError_1 = require("../Log/LarkError");
var LarkLogger_1 = require("../Log/LarkLogger");
var ApplicationDto = require("./ApplicationDto");
var logger = new LarkLogger_1.LarkLogger("Core/Application");
var Application = /** @class */ (function () {
    function Application() {
    }
    Application.getConstant = function () {
        return this.constant;
    };
    /**
     * 多次调用，不防异步
     */
    Application.ready = function () {
        if (this.init === false) {
            if (process.env.debug) {
                this.debug = true;
            }
            else {
                this.debug = false;
            }
            if (fs.existsSync("./config/constant.json")) {
                this.constant = Object.assign(this.constant, fs.readJSONSync("./config/constant.json"));
            }
            var packageInfo = fs.readJSONSync("./package.json");
            this.appName = packageInfo.name;
            this.appVersion = packageInfo.version;
            this.host = packageInfo.host;
            this.port = packageInfo.port;
            this.version = packageInfo.version;
            // logger.log(
            //     "Application.init",
            //     JSON.stringify({
            //         appName: this.appName,
            //         appVersion: this.appVersion,
            //         debug: this.debug,
            //         host: this.host,
            //         port: this.port
            //     }),
            // )
            this.init = true;
        }
        return this;
    };
    Application.getVersion = function () {
        return this.version;
    };
    Application.getPackageName = function () {
        return this.appName;
    };
    /**
     * 获取完整的带版本号的包名
     * @example kemao-lark@6.0.0
     */
    Application.getFullPackageName = function () {
        return this.appName + "@" + this.version;
    };
    Application.getLarkServer = function () {
        var server = "https://lark.kemao.com";
        if (this.isDebug()) {
            server = server.replace(".kemao.", ".kemao-dev.");
        }
        return server;
    };
    Application.getRemoteServer = function () {
        if (this.host === null || typeof this.host === "undefined") {
            throw new Error("没有配置host，请配置host。或使用getLocalServer");
        }
        var host = this.host;
        if (this.isDebug()) {
            host = host.replace(".kemao.", ".kemao-dev.");
        }
        return "https://" + host;
    };
    Application.getLocalServer = function () {
        if (this.port === null) {
            throw new LarkError_1.LarkError("Application初始化失败或未设置无法读取port");
        }
        return "https://127.0.0.1:" + this.port;
    };
    Application.setDebug = function (debug) {
        this.debug = debug;
    };
    Application.isDebug = function () {
        return this.debug;
    };
    Application.appName = null;
    Application.appVersion = null;
    Application.init = false;
    Application.debug = false;
    Application.port = null;
    Application.host = null;
    Application.version = null;
    Application.constant = ApplicationDto;
    return Application;
}());
exports.Application = Application;
Application.ready();
