"use strict";
/*! *****************************************************************************
Copyright (c) JDJR Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0
Author: kemao 2018-11-29 18:28:33

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
exports.__esModule = true;
var tslib_1 = require("tslib");
var fs = require("fs-extra");
var HTTP = require("http");
var HTTPS = require("https");
var jsdom_1 = require("jsdom");
var SHA1 = require("sha1");
var URL = require("url");
var LarkError_1 = require("../Log/LarkError");
var LarkLogger_1 = require("../Log/LarkLogger");
var FileSystem_1 = require("../Storage/FileSystem");
var LarkMath_1 = require("../Utils/LarkMath");
var HTTP_STATUS_1 = require("./HTTP_STATUS");
/**
 * 支持保持server
 * 支持保存cookie
 * 支持编码纠正
 * 支持异常处理
 * 支持超时机制
 */
var logger = new LarkLogger_1.LarkLogger("Http/HttpClient");
var HttpClient = /** @class */ (function () {
    function HttpClient() {
        this.hostname = null;
        this.cookies = null;
    }
    HttpClient.download = function (url, filename, headers) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var buffer;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(fs.existsSync(filename) === false)) return [3 /*break*/, 2];
                        logger.info("download.start", url);
                        return [4 /*yield*/, this.get(url, headers)];
                    case 1:
                        buffer = _a.sent();
                        logger.info("download.end.write", url);
                        fs.outputFileSync(filename, buffer);
                        logger.info("download.end.success", { filename: filename, url: url });
                        return [3 /*break*/, 3];
                    case 2:
                        logger.warn("文件已存在，跳过下载", { filename: filename, url: url });
                        _a.label = 3;
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    HttpClient.getFileSHA1ByURL = function (url, headers) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var buffer, sha1, error_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4 /*yield*/, this.get(url, headers)];
                    case 1:
                        buffer = _a.sent();
                        sha1 = SHA1(buffer);
                        return [2 /*return*/, { sha1: sha1, buffer: buffer }];
                    case 2:
                        error_1 = _a.sent();
                        logger.error("getImageSHA1", error_1);
                        return [2 /*return*/, { sha1: null }];
                    case 3: return [2 /*return*/];
                }
            });
        });
    };
    HttpClient.getJSON = function (url, headers) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var buf;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.get(url, headers)];
                    case 1:
                        buf = _a.sent();
                        return [2 /*return*/, JSON.parse(buf.toString())];
                }
            });
        });
    };
    HttpClient.getDOM = function (url, headers) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var html, dom;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (url === null) {
                            throw new LarkError_1.LarkError("url不能为null");
                        }
                        return [4 /*yield*/, this.get(url, headers)];
                    case 1:
                        html = _a.sent();
                        dom = new jsdom_1.JSDOM(html).window.document;
                        return [2 /*return*/, dom];
                }
            });
        });
    };
    /**
     * 增加编码转换功能，以便支持非utf8的内容的获取
     */
    HttpClient.getText = function (url, headers) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var buf;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.get(url, headers)];
                    case 1:
                        buf = _a.sent();
                        return [2 /*return*/, buf.toString()];
                }
            });
        });
    };
    HttpClient.get = function (url, headers) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var response, location_1, res;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getResponse(url, headers)];
                    case 1:
                        response = _a.sent();
                        if (!(response.statusCode === HTTP_STATUS_1.HTTP_STATUS.MOVE_TEMPORARILY || response.statusCode === HTTP_STATUS_1.HTTP_STATUS.MOVED_PERMANENTLY)) return [3 /*break*/, 3];
                        location_1 = response.headers.location;
                        logger.info("请求重定向", { location: location_1 });
                        if (typeof location_1 !== "string" || location_1 === "") {
                            throw (new LarkError_1.LarkError("重定向状态下未获得重定向地址", { headers: response.headers }));
                        }
                        return [4 /*yield*/, this.get(location_1, headers)];
                    case 2:
                        res = _a.sent();
                        return [2 /*return*/, res.body];
                    case 3: return [2 /*return*/, response.body];
                }
            });
        });
    };
    HttpClient.getResponse = function (url, headers) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        var urlParse = URL.parse(url);
                        var Request = HTTP.request;
                        if (urlParse.protocol === "https:" || Number(urlParse.port) === 443) {
                            urlParse.protocol = "https:";
                            Request = HTTPS.request;
                        }
                        headers = Object.assign({
                            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) " +
                                "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.101 Safari/537.36"
                        }, headers);
                        var requestData = {
                            protocol: urlParse.protocol,
                            port: urlParse.port,
                            method: "GET",
                            host: urlParse.hostname,
                            path: urlParse.path,
                            headers: headers,
                            rejectUnauthorized: false
                        };
                        logger.info({ urlParse: urlParse, requestData: requestData });
                        var req = Request(requestData, function (res) {
                            var total = 0;
                            var arrayBuffer = [];
                            logger.log("STATUS: " + res.statusCode);
                            logger.log("HEADERS: " + JSON.stringify(res.headers));
                            res.on("data", function (chunk) {
                                total += chunk.length;
                                logger.single("BODY:", LarkMath_1.LarkMath.getPercent(total, Number(res.headers["content-length"])), _this.getProcessRate(Number(res.headers["content-length"]), total));
                                arrayBuffer.push(chunk);
                            });
                            res.on("end", function () {
                                logger.log("response end");
                                resolve({
                                    statusCode: res.statusCode,
                                    headers: res.headers,
                                    body: Buffer.concat(arrayBuffer)
                                });
                            });
                        });
                        req.on("error", function (e) {
                            logger.error("problem with request: " + e.message);
                            reject(new LarkError_1.LarkError(e.message, e));
                        });
                        req.end();
                    })];
            });
        });
    };
    HttpClient.getProcessRate = function (length, total) {
        if (isNaN(length)) {
            return FileSystem_1.FileSystem.getVisualFileSize(total);
        }
        return FileSystem_1.FileSystem.getVisualFileSize(total) + "/" + FileSystem_1.FileSystem.getVisualFileSize(length);
    };
    /**
     * 推送buffer
     * @param url
     * @param data Buffer|FormData|JSON
     * @param headers
     */
    HttpClient.post = function (url, data, headers) {
        if (headers === void 0) { headers = null; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        if (headers === null) {
                            headers = {};
                        }
                        headers = Object.assign({
                            "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) " +
                                "AppleWebKit/537.36 (KHTML, like Gecko) Chrome/60.0.3112.101 Safari/537.36"
                        }, headers);
                        headers["Content-Length"] = Buffer.byteLength(data);
                        var urlParse = URL.parse(url);
                        var Request = HTTP.request;
                        if (urlParse.protocol === "https:" || Number(urlParse.port) === 443) {
                            urlParse.protocol = "https:";
                            Request = HTTPS.request;
                        }
                        var postData = {
                            protocol: urlParse.protocol,
                            port: urlParse.port,
                            method: "POST",
                            host: urlParse.hostname,
                            path: urlParse.path,
                            headers: headers
                        };
                        var req = Request(postData, function (res) {
                            var arrayBuffer = [];
                            logger.log("STATUS: " + res.statusCode);
                            logger.log("HEADERS: " + JSON.stringify(res.headers));
                            // res.setEncoding('utf8');
                            res.on("data", function (chunk) {
                                // logger.log(`BODY: ${chunk}`);
                                arrayBuffer.push(chunk);
                            });
                            res.on("end", function () {
                                logger.log("No more data in response.");
                                resolve(Buffer.concat(arrayBuffer));
                            });
                        });
                        req.on("error", function (e) {
                            logger.error("problem with request: " + e.message);
                            reject(new LarkError_1.LarkError(e.message, e));
                        });
                        // write data to request body
                        req.write(data);
                        req.end();
                    })];
            });
        });
    };
    HttpClient.topic = "Lark/Http/HttpClient";
    return HttpClient;
}());
exports.HttpClient = HttpClient;
