"use strict";
/*! *****************************************************************************
Copyright (c) JDJR Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0
Author: kemao 2018-12-04 12:16:45

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
exports.__esModule = true;
var tslib_1 = require("tslib");
var fs = require("fs-extra");
var mongodb_1 = require("mongodb");
var LarkLogger_1 = require("../Log/LarkLogger");
var logger = new LarkLogger_1.LarkLogger("MongoDB");
var MongoDB = /** @class */ (function () {
    function MongoDB(db) {
        this.db = null;
        this.db = db;
    }
    MongoDB.init = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                if (MongoDB.mongodbList === null) {
                    MongoDB.mongodbList = fs.readJSONSync("./config/mongodb.json");
                }
                return [2 /*return*/];
            });
        });
    };
    MongoDB.prototype.getDB = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var db, config, url, options;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        db = MongoDB.pool.get(this.db);
                        if (!(typeof db === "undefined")) return [3 /*break*/, 2];
                        return [4 /*yield*/, MongoDB.init()];
                    case 1:
                        _a.sent();
                        config = MongoDB.mongodbList.dbMap[this.db];
                        if (config === undefined) {
                            throw new Error("\u8BE5\u6570\u636E\u5E93\u672A\u914D\u7F6E\uFF0C\u8BF7\u6DFB\u52A0\u914D\u7F6Econfig/mongodb.json->dbMap." + this.db);
                        }
                        url = null;
                        options = {
                            autoReconnect: true,
                            poolSize: 1,
                            useNewUrlParser: true
                        };
                        if (typeof config === "string") {
                            url = config;
                        }
                        else {
                            url = config.url;
                            if (config.option) {
                                options = config.option;
                            }
                        }
                        logger.log("初始化数据库", this.db, url.replace(/kemao|root|maoke/, "****").replace(/\d|\./g, "*"));
                        // 推入了一个Promise对象
                        MongoDB.pool.set(this.db, mongodb_1.MongoClient.connect(url, options));
                        _a.label = 2;
                    case 2: return [4 /*yield*/, MongoDB.pool.get(this.db)];
                    case 3:
                        // 等待直到它完成
                        db = _a.sent();
                        return [2 /*return*/, db.db(this.db)];
                }
            });
        });
    };
    MongoDB.prototype.getCollection = function (collectionName) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var db, collection;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getDB()];
                    case 1:
                        db = _a.sent();
                        return [4 /*yield*/, db.collection(collectionName)];
                    case 2:
                        collection = _a.sent();
                        return [2 /*return*/, collection];
                }
            });
        });
    };
    MongoDB.prototype.getSequence = function (key, collectionName) {
        if (collectionName === void 0) { collectionName = "Sequence"; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var collection, record;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getCollection(collectionName)];
                    case 1:
                        collection = _a.sent();
                        return [4 /*yield*/, collection.findOneAndUpdate({
                                key: key
                            }, {
                                $inc: {
                                    value: 1
                                },
                                $set: {
                                    updateDatetime: new Date()
                                }
                            }, {
                                upsert: true
                            })];
                    case 2:
                        record = _a.sent();
                        if (record.value === null) {
                            return [2 /*return*/, 0];
                        }
                        return [2 /*return*/, record.value.value];
                }
            });
        });
    };
    MongoDB.prototype.close = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var dbPromise, db;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        dbPromise = MongoDB.pool.get(this.db);
                        MongoDB.pool["delete"](this.db);
                        return [4 /*yield*/, dbPromise];
                    case 1:
                        db = _a.sent();
                        logger.log("关闭数据库", this.db);
                        return [4 /*yield*/, db.close()];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    MongoDB.pool = new Map();
    MongoDB.mongodbList = null;
    return MongoDB;
}());
exports.MongoDB = MongoDB;
