"use strict";
/*! *****************************************************************************
Copyright (c) JDJR Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0
Author: kemao 2018-12-04 12:17:33

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
exports.__esModule = true;
var tslib_1 = require("tslib");
var mongodb_1 = require("mongodb");
var Path = require("path");
var LarkError_1 = require("../../Log/LarkError");
var LarkLogger_1 = require("../../Log/LarkLogger");
var LarkBuffer_1 = require("../../Utils/LarkBuffer");
var FileSystem_1 = require("../FileSystem");
var MongoDB_1 = require("../MongoDB");
var logger = new LarkLogger_1.LarkLogger("Mongodb/MongoDBGridFS");
var MongoDBGridFS = /** @class */ (function () {
    /**
     *
     * @param dbName
     * @param bucketName like collectionName
     */
    function MongoDBGridFS(dbName, bucketName) {
        this.dbName = null;
        this.bucketName = null;
        this.mdb = null;
        this.dbName = dbName;
        this.bucketName = bucketName;
        this.mdb = new MongoDB_1.MongoDB(this.dbName);
    }
    MongoDBGridFS.prototype.readFile = function (filename) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var stream, buffer;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.readFileStream(filename)];
                    case 1:
                        stream = _a.sent();
                        return [4 /*yield*/, LarkBuffer_1.LarkBuffer.readBufferByStream(stream)];
                    case 2:
                        buffer = _a.sent();
                        return [2 /*return*/, buffer];
                }
            });
        });
    };
    MongoDBGridFS.prototype.readFileStream = function (filename) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var db, bucket, stream;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getDB()];
                    case 1:
                        db = _a.sent();
                        bucket = new mongodb_1.GridFSBucket(db, { bucketName: this.bucketName });
                        stream = bucket.openDownloadStreamByName(filename);
                        return [2 /*return*/, stream];
                }
            });
        });
    };
    /**
     * 获取文件状态信息
     */
    MongoDBGridFS.prototype.getFileStat = function (filename) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var collection, record;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.mdb.getCollection(this.bucketName + ".files")];
                    case 1:
                        collection = _a.sent();
                        return [4 /*yield*/, collection.findOne({ filename: filename })];
                    case 2:
                        record = _a.sent();
                        return [2 /*return*/, record];
                }
            });
        });
    };
    /**
     * 通过文件名删除文件
     * @param filename
     */
    MongoDBGridFS.prototype.removeFile = function (filename) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        var collection, record, db, bucket;
                        return tslib_1.__generator(this, function (_a) {
                            switch (_a.label) {
                                case 0: return [4 /*yield*/, this.mdb.getCollection(this.bucketName + ".files")];
                                case 1:
                                    collection = _a.sent();
                                    return [4 /*yield*/, collection.findOne({ filename: filename })];
                                case 2:
                                    record = _a.sent();
                                    return [4 /*yield*/, this.getDB()];
                                case 3:
                                    db = _a.sent();
                                    bucket = new mongodb_1.GridFSBucket(db, { bucketName: this.bucketName });
                                    bucket["delete"](record._id, function (error) {
                                        if (error !== null) {
                                            reject(error);
                                        }
                                        else {
                                            resolve();
                                        }
                                    });
                                    return [2 /*return*/];
                            }
                        });
                    }); })];
            });
        });
    };
    /**
     * 追加文件的元数据
     * @param filename
     * @param meta
     */
    MongoDBGridFS.prototype.appendFileMeta = function (filename, metadata) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var collection, record;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.mdb.getCollection(this.bucketName + ".files")];
                    case 1:
                        collection = _a.sent();
                        return [4 /*yield*/, collection.findOne({ filename: filename })];
                    case 2:
                        record = _a.sent();
                        record.metadata = Object.assign(record.metadata, metadata);
                        logger.info("appendFileMeta", { record: record });
                        return [4 /*yield*/, collection.updateOne({ _id: record._id }, {
                                $set: record
                            })];
                    case 3:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    MongoDBGridFS.prototype.outputFile = function (filename, buffer) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var stat;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.getFileStat(filename)];
                    case 1:
                        stat = _a.sent();
                        if (!(stat !== null)) return [3 /*break*/, 3];
                        logger.warn("outputFile.覆盖", filename, buffer.length);
                        return [4 /*yield*/, this.removeFile(filename)];
                    case 2:
                        _a.sent();
                        _a.label = 3;
                    case 3: return [4 /*yield*/, this.writeFile(filename, buffer)];
                    case 4:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * 根据文件后缀名自动确定contentType
     * @param filename
     * @param buffer
     */
    MongoDBGridFS.prototype.writeFile = function (filename, buffer, metadata) {
        if (metadata === void 0) { metadata = null; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        var stat, db, bucket, writeStream;
                        return tslib_1.__generator(this, function (_a) {
                            switch (_a.label) {
                                case 0:
                                    logger.info("writeFile", filename, buffer.length);
                                    return [4 /*yield*/, this.getFileStat(filename)];
                                case 1:
                                    stat = _a.sent();
                                    if (stat !== null) {
                                        reject(new LarkError_1.LarkError("文件已经存在", { stat: stat, filename: filename }, 1535546953353));
                                        return [2 /*return*/];
                                    }
                                    return [4 /*yield*/, this.getDB()];
                                case 2:
                                    db = _a.sent();
                                    bucket = new mongodb_1.GridFSBucket(db, { bucketName: this.bucketName });
                                    writeStream = bucket.openUploadStream(filename, {
                                        contentType: FileSystem_1.FileSystem.getMIMEByExtname(Path.extname(filename)),
                                        metadata: metadata
                                    });
                                    writeStream.write(buffer, function (error) {
                                        if (error === null) {
                                        }
                                        else {
                                            logger.error("writeFile.error", { error: error });
                                            reject(error);
                                        }
                                    });
                                    writeStream.end(function (endError) {
                                        if (endError === null) {
                                            resolve();
                                        }
                                        else {
                                            reject(endError);
                                        }
                                    });
                                    return [2 /*return*/];
                            }
                        });
                    }); })];
            });
        });
    };
    MongoDBGridFS.prototype.writeFileStream = function (filename, readStream) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                        var db, bucket, writeStream;
                        return tslib_1.__generator(this, function (_a) {
                            switch (_a.label) {
                                case 0: return [4 /*yield*/, this.getDB()];
                                case 1:
                                    db = _a.sent();
                                    bucket = new mongodb_1.GridFSBucket(db, { bucketName: this.bucketName });
                                    writeStream = bucket.openUploadStream(filename);
                                    readStream.on("error", function (error) {
                                        reject(error); // 流读取失败
                                    });
                                    writeStream.on("error", function (error) {
                                        reject(error); // 流写入失败
                                    });
                                    writeStream.on("end", function () {
                                        resolve(); // 结束
                                    });
                                    readStream.pipe(writeStream);
                                    return [2 /*return*/];
                            }
                        });
                    }); })];
            });
        });
    };
    MongoDBGridFS.prototype.close = function () {
        this.mdb.close();
    };
    MongoDBGridFS.prototype.getDB = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.mdb.getDB()];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    return MongoDBGridFS;
}());
exports.MongoDBGridFS = MongoDBGridFS;
