"use strict";
/*! *****************************************************************************
Copyright (c) JDJR Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0
Author: kemao 2018-12-04 12:17:53

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
exports.__esModule = true;
var tslib_1 = require("tslib");
var zlib = require("zlib");
var LarkError_1 = require("../Log/LarkError");
exports.ERROR_READ_BUFFER_FAIL = { message: "读取流数据失败", code: 1534907805239 };
exports.ERROR_MAX_BODY_SIZE_LIMIT = { message: "超出最大可接收的Buffer长度", code: 1534907814604 };
var LarkBuffer = /** @class */ (function () {
    function LarkBuffer() {
    }
    LarkBuffer.getBufferByStream = function (stream, maxBodySize) {
        if (maxBodySize === void 0) { maxBodySize = null; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4 /*yield*/, this.readBufferByStream(stream, maxBodySize)];
                    case 1: return [2 /*return*/, _a.sent()];
                }
            });
        });
    };
    /**
     * 通过stream获取其buffer
     * @param maxBodySize 是否控制最大大小，如果超过最大大小。触发异常中断读取
     * 是否增加超时机制
     * @deprecated use getBufferByStream
     */
    LarkBuffer.readBufferByStream = function (stream, maxBodySize) {
        if (maxBodySize === void 0) { maxBodySize = null; }
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        var buffer = [];
                        var size = 0;
                        stream.on("error", function (error) {
                            reject(new LarkError_1.LarkError(exports.ERROR_READ_BUFFER_FAIL.message, { error: error }));
                        });
                        stream.on("data", function (chunk) {
                            size += chunk.length;
                            if (typeof maxBodySize === "number" && size > maxBodySize) {
                                reject(new LarkError_1.LarkError(exports.ERROR_MAX_BODY_SIZE_LIMIT.message, { size: size, maxBodySize: maxBodySize }));
                            }
                            else {
                                buffer.push(chunk);
                            }
                        });
                        stream.on("end", function () {
                            resolve(Buffer.concat(buffer));
                        });
                    })];
            });
        });
    };
    LarkBuffer.GZIP = function (buffer) {
        return zlib.gzipSync(buffer);
    };
    LarkBuffer.UNGZIP = function (buffer) {
        return zlib.gunzipSync(buffer);
    };
    return LarkBuffer;
}());
exports.LarkBuffer = LarkBuffer;
