"use strict";
/*! *****************************************************************************
Copyright (c) JDJR Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0
Author: kemao 2018-12-04 12:18:18

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
exports.__esModule = true;
var Lodash = require("lodash");
var LarkArray_1 = require("./LarkArray");
exports.UNDEFINED = "undefined";
var LarkObject = /** @class */ (function () {
    function LarkObject() {
    }
    LarkObject.isUndefined = function (obj) {
        return typeof obj === exports.UNDEFINED;
    };
    LarkObject.isNull = function (obj) {
        return obj === null;
    };
    LarkObject.isNullOrIsUndefined = function (obj) {
        return LarkObject.isUndefined(obj) || LarkObject.isNull(obj);
    };
    /**
     * 对一个js对象进行深度排序，按其名称字母
     * @param obj
     */
    LarkObject.sort = function (obj) {
        if (obj instanceof Array) {
            return obj;
        }
        var newObj = {};
        var keys = Object.keys(obj);
        keys = keys.sort();
        keys.map(function (key) {
            if (obj[key] instanceof Object && (typeof obj[key] === "function") === false) {
                newObj[key] = LarkObject.sort(obj[key]);
            }
            else {
                newObj[key] = obj[key];
            }
        });
        return newObj;
    };
    /**
     * 合并两个对象，并进行深层重复值去重
     * @param obj
     */
    LarkObject.merge = function (obj, option) {
        if (option === void 0) { option = { clean: true, concatArray: true }; }
        var defaultOption = {
            clean: true, concatArray: true
        };
        option = Object.assign(defaultOption, option);
        var data = Lodash.mergeWith(obj.from, obj.to, option.concatArray ? function (objValue, srcValue) {
            if (objValue instanceof Array) {
                var arr = Lodash.union(objValue.concat(srcValue));
                return arr;
            }
        } : null);
        if (option.clean) {
            data = LarkObject.cleanObjectData(data);
        }
        return data;
    };
    /**
     * 清除对象里面的无用数据
     * @param obj
     */
    LarkObject.cleanObjectData = function (obj) {
        for (var i in obj) {
            if (obj.hasOwnProperty(i)) {
                if (obj[i] instanceof Array) {
                    obj[i] = LarkArray_1.LarkArray.cleanArrayNull(obj[i]);
                }
                else if (typeof obj[i] === "object") {
                    obj[i] = LarkObject.cleanObjectData(obj[i]);
                }
            }
        }
        return obj;
    };
    return LarkObject;
}());
exports.LarkObject = LarkObject;
