"use strict";
/*! *****************************************************************************
Copyright (c) JDJR Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0
Author: kemao 2018-11-29 19:34:35

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
exports.__esModule = true;
var tslib_1 = require("tslib");
var EJS = require("ejs");
var HTTP_STATUS_1 = require("../../Http/HTTP_STATUS");
var LarkError_1 = require("../../Log/LarkError");
var Controller_1 = require("../Controller");
var LarkResponse_1 = require("../LarkResponse");
var PageController = /** @class */ (function (_super) {
    tslib_1.__extends(PageController, _super);
    function PageController() {
        var _this = _super.call(this) || this;
        _this.title = null;
        // body下文的脚本
        _this.script = [];
        _this.inlineScript = [];
        // 整站样式的位置
        _this.style = [];
        _this.inlineStyle = [];
        // head当中插入的脚本
        // "libs/vendor"
        _this.headScript = [];
        _this.meta = [];
        _this.link = [];
        _this.viewport();
        return _this;
    }
    PageController.prototype.addStyle = function (url, remote) {
        if (remote === void 0) { remote = false; }
        this.style.push(url);
        return this;
    };
    PageController.prototype.addInlineStyle = function (style) {
        if (typeof style === "object") {
            style = style._getCss();
        }
        this.inlineStyle.push(style);
    };
    PageController.prototype.addInlineScript = function (script) {
        this.inlineScript.push(script);
    };
    PageController.prototype.addLink = function (link) {
        this.link.push(link);
        return this;
    };
    PageController.prototype.addMeta = function (meta) {
        this.meta.push(meta);
        return this;
    };
    PageController.prototype.addHeadScript = function (script) {
        this.headScript.push(script);
    };
    PageController.prototype.viewport = function () {
        this.meta.push({
            content: "width=device-width, initial-scale=1, maximum-scale=1",
            name: "viewport"
        });
    };
    PageController.prototype.addScript = function (href, remote) {
        if (remote === void 0) { remote = false; }
        this.script.push(href);
        return this;
    };
    /**
     * 渲染网页
     * @param view - 模版
     * @param data - 渲染用的数据
     * @param userDefaultHTMLTemplate 是否使用缺省的页面模版
     */
    PageController.prototype.render = function (view, data, userDefaultHTMLTemplate) {
        if (view === void 0) { view = ""; }
        if (data === void 0) { data = {}; }
        if (userDefaultHTMLTemplate === void 0) { userDefaultHTMLTemplate = true; }
        if (data === null) {
            data = {};
        }
        data.__page = {
            title: this.title,
            style: this.style,
            meta: this.meta,
            script: this.script,
            headScript: this.headScript,
            link: this.link,
            inlineStyle: this.inlineStyle,
            inlineScript: this.inlineScript
        };
        // console.log("render", view, data)
        var html = null;
        var bodyHTML = null;
        try {
            bodyHTML = EJS.render(view, data, { rmWhitespace: true }).replace(/\n/g, "");
        }
        catch (error) {
            throw new LarkError_1.LarkError("模版渲染失败", { error: error, data: data, view: view }, 1535109244934);
        }
        // 使用缺省的母版页
        if (userDefaultHTMLTemplate) {
            html = EJS.render(require("./PageController/PageController.ejs"), { body: bodyHTML, __page: data.__page }, { rmWhitespace: true })
                .replace(/\n/g, "");
        }
        else {
            html = bodyHTML;
        }
        return new LarkResponse_1.LarkResponse(html, HTTP_STATUS_1.HTTP_STATUS.OK, { "Content-Type": "text/html;charset=UTF-8" });
    };
    return PageController;
}(Controller_1.Controller));
exports.PageController = PageController;
