"use strict";
/*! *****************************************************************************
Copyright (c) JDJR Corporation. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License"); you may not use
this file except in compliance with the License. You may obtain a copy of the
License at http://www.apache.org/licenses/LICENSE-2.0
Author: kemao 2018-11-29 16:22:25

THIS CODE IS PROVIDED ON AN *AS IS* BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION ANY IMPLIED
WARRANTIES OR CONDITIONS OF TITLE, FITNESS FOR A PARTICULAR PURPOSE,
MERCHANTABLITY OR NON-INFRINGEMENT.

See the Apache Version 2.0 License for specific language governing permissions
and limitations under the License.
***************************************************************************** */
exports.__esModule = true;
var tslib_1 = require("tslib");
var fs = require("fs-extra");
var HTTP = require("http");
var QueryString = require("querystring");
var URL = require("url");
var HTTP_STATUS_1 = require("../Http/HTTP_STATUS");
var LarkError_1 = require("../Log/LarkError");
var LarkLogger_1 = require("../Log/LarkLogger");
var LarkBuffer_1 = require("../Utils/LarkBuffer");
var ServerStaticController_1 = require("./Controller/ServerStaticController");
var LarkRequest_1 = require("./LarkRequest");
var LarkResponse_1 = require("./LarkResponse");
var WebServerDto = require("./WebServerDto");
var WebServerDto_1 = require("./WebServerDto");
var logger = new LarkLogger_1.LarkLogger("WebServer");
/**
 * @deprecated 应该使用HttpServer取代WebServer
 */
var WebServer = /** @class */ (function () {
    function WebServer() {
        this.httpServer = null;
        this.httpsServer = null;
        this.websocket = null;
        this.CONFIG = WebServerDto;
        this.controllerList = [];
    }
    WebServer.prototype.initCONFIG = function () {
        if (fs.existsSync("./config/web.json")) {
            var config = fs.readJSONSync("config/web.json");
            this.CONFIG = Object.assign(this.CONFIG, config);
        }
    };
    WebServer.prototype.getRouterLarkResponse = function (request) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var hostname, userURI, _i, _a, Controller, controller, controllerURI, uriStringHit, uriRegExpHit, uriHit, hasController;
            return tslib_1.__generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        hostname = request.headers.host.split(":")[0];
                        userURI = request.url.split("?")[0];
                        _i = 0, _a = this.controllerList;
                        _b.label = 1;
                    case 1:
                        if (!(_i < _a.length)) return [3 /*break*/, 5];
                        Controller = _a[_i];
                        controller = new Controller();
                        if (!(hostname === controller.hostname || controller.hostname === null)) return [3 /*break*/, 4];
                        logger.log("hostname.hit", { hostname: hostname });
                        controllerURI = null;
                        if (Controller.URI === undefined) {
                            controllerURI = controller.uri;
                        }
                        else {
                            controllerURI = Controller.URI;
                        }
                        uriStringHit = (typeof controllerURI === "string" && controllerURI === userURI);
                        uriRegExpHit = controllerURI instanceof RegExp && controllerURI.test(userURI);
                        uriHit = uriStringHit || uriRegExpHit;
                        if (!uriHit) return [3 /*break*/, 4];
                        hasController = typeof controller[request.method.toLowerCase()] === "function";
                        logger.log("uriHit", { uriHit: uriHit, controllerURI: controllerURI, userURI: userURI, hasController: hasController });
                        if (!hasController) return [3 /*break*/, 3];
                        return [4 /*yield*/, this.getParserLarkResponse(Controller, request, userURI)];
                    case 2: return [2 /*return*/, _b.sent()];
                    case 3: throw new LarkError_1.LarkError(WebServerDto_1.ERROR_UNSUPPORT_METHOD.message, {
                        method: request.method.toLowerCase(),
                        uriStringHit: uriStringHit,
                        hasController: hasController,
                        uriRegExpHit: uriRegExpHit,
                        uriHit: uriHit,
                        userURI: userURI,
                        controllerURI: controllerURI
                    }, WebServerDto_1.ERROR_UNSUPPORT_METHOD.code);
                    case 4:
                        _i++;
                        return [3 /*break*/, 1];
                    case 5: throw new LarkError_1.LarkError(WebServerDto_1.ERROR_ROUTER_NOT_FOUND.message, null, WebServerDto_1.ERROR_ROUTER_NOT_FOUND.code);
                }
            });
        });
    };
    WebServer.prototype.router = function (request, response) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var buffer, header, status, larkResponse, error_1;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        logger.log(request.headers, request.method, request.httpVersion, request.url);
                        buffer = null;
                        header = null;
                        status = null;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4 /*yield*/, this.getRouterLarkResponse(request)];
                    case 2:
                        larkResponse = _a.sent();
                        logger.log({ larkResponse: larkResponse });
                        // 响应response
                        header = larkResponse.header;
                        // 响应内容
                        buffer = larkResponse.getData();
                        status = larkResponse.status;
                        return [3 /*break*/, 4];
                    case 3:
                        error_1 = _a.sent();
                        logger.log(error_1);
                        status = 500;
                        header = {
                            "Content-Type": "application/json"
                        };
                        buffer = Buffer.from(JSON.stringify({ ok: false, message: "服务器异常", error: error_1 }));
                        if (error_1 instanceof LarkError_1.LarkError) {
                            switch (error_1.code) {
                                case WebServerDto_1.ERROR_ROUTER_NOT_FOUND.code: // 未命中
                                    status = HTTP_STATUS_1.HTTP_STATUS.NOT_FOUND;
                                    buffer = Buffer.from(JSON.stringify({ ok: false, message: WebServerDto_1.ERROR_ROUTER_NOT_FOUND.message, error: error_1 }));
                                    break;
                                case WebServerDto_1.ERROR_UNSUPPORT_METHOD.code: // 不支持的请求
                                    status = HTTP_STATUS_1.HTTP_STATUS.METHOD_NOT_ALLOWED;
                                    buffer = Buffer.from(JSON.stringify({ ok: false, message: WebServerDto_1.ERROR_UNSUPPORT_METHOD.message, error: error_1 }));
                                    header.Allow = error_1.data.method;
                                    break;
                                case LarkBuffer_1.ERROR_MAX_BODY_SIZE_LIMIT.code: // 超过能接收的数据大小
                                    status = HTTP_STATUS_1.HTTP_STATUS.REQUEST_ENTITY_TOO_LARGE;
                                    buffer = Buffer.from(JSON.stringify({ ok: false, message: LarkBuffer_1.ERROR_MAX_BODY_SIZE_LIMIT.message, error: error_1 }));
                                    break;
                                default:
                                    buffer = Buffer.from(JSON.stringify({ ok: false, error: error_1 }));
                            }
                        }
                        return [3 /*break*/, 4];
                    case 4:
                        if (LarkRequest_1.LarkRequest.hasAcceptEncodingGZIP(request.headers)) {
                            buffer = LarkBuffer_1.LarkBuffer.GZIP(buffer);
                            header["Content-Encoding"] = "gzip";
                        }
                        response.writeHead(status, header);
                        response.end(buffer);
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * 启动服务
     */
    WebServer.prototype.startup = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        this.controllerList.push(ServerStaticController_1.ServerStaticController);
                        return [4 /*yield*/, this.initCONFIG()];
                    case 1:
                        _a.sent();
                        return [4 /*yield*/, this.listenHTTP()];
                    case 2:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    WebServer.prototype.listenHTTP = function () {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                if (this.CONFIG.PORT === null) {
                    return [2 /*return*/];
                }
                return [2 /*return*/, new Promise(function (resolve, reject) {
                        _this.httpServer = HTTP.createServer(function (req, res) {
                            _this.router(req, res);
                        });
                        _this.httpServer.listen(_this.CONFIG.PORT, function (error) {
                            if (error) {
                                reject(error);
                            }
                            else {
                                logger.log("listen http", "http://127.0.0.1:" + _this.CONFIG.PORT);
                                resolve();
                            }
                        });
                    })];
            });
        });
    };
    WebServer.prototype.addController = function (controller) {
        this.controllerList.push(controller);
        return this;
    };
    WebServer.prototype.getParserLarkResponse = function (Controller, request, userURI) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var method, url, query, params, cookie, headers, controller, larkResponse;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        method = request.method.toLowerCase();
                        url = URL.parse(request.url);
                        query = QueryString.parse(url.query);
                        params = null;
                        cookie = request.headers.cookie;
                        headers = request.headers;
                        controller = new Controller();
                        logger.log("getParserLarkResponse", { method: method, url: url, query: query });
                        return [4 /*yield*/, controller[method](request, { url: url, method: method, query: query, params: params, cookie: cookie, headers: headers, uri: userURI })];
                    case 1:
                        larkResponse = _a.sent();
                        if (larkResponse === undefined) {
                            larkResponse = new LarkResponse_1.LarkResponse(); // 如果方法没有返回消息则自动分配一个响应
                        }
                        return [2 /*return*/, larkResponse];
                }
            });
        });
    };
    return WebServer;
}());
exports.WebServer = WebServer;
